precision highp float;

attribute vec4 position;
attribute vec4 normal;

varying vec2 texcoordVarying;

// xyz: undefined, w: vertex alpha
varying vec4 diffuseVarying;

uniform vec4 viewport;
uniform vec4 pixelRotation;
uniform vec4 diffuseColor;

//xy: pixel translation, zw : pixelScale
uniform vec4 pixelTranslationScale;
//xy: postPixelTranslation, zw : scale
uniform vec4 postPixelTranslation;

vec3 rotate_vector( vec4 quat, vec3 vec )
{
	return vec + 2.0 * cross( cross( vec, quat.xyz ) + quat.w * vec, quat.xyz );
}

void main()
{
    texcoordVarying = normal.zw;
    
    // xyz: undefined, w: vertex alpha
    diffuseVarying = vec4( diffuseColor.xyz, position.w * diffuseColor.w );
    
    vec4 currentPosition = vec4( ( position.x / viewport.z ) * 2.0 - 1.0,
                                 ( ( viewport.w - position.y ) / viewport.w ) * 2.0 - 1.0,
                                 -1.0,
                                 1.0 );
    
    // calculates ndc space labelposition
	vec3 translation = rotate_vector( pixelRotation, vec3( normal.xy, 0.0) );
    translation.xy *= pixelTranslationScale.zw;

    translation += rotate_vector( pixelRotation, vec3( pixelTranslationScale.xy, 0.0 ) );
	
//    translation.xy *= postPixelTranslation.zw;
    translation.xy += postPixelTranslation.xy;

	gl_Position.x = currentPosition.x + ( translation.x / viewport.z ) * 2.0;
	gl_Position.y = currentPosition.y - ( translation.y / viewport.w ) * 2.0 ;
    gl_Position.zw = currentPosition.zw;

}
