#version 450

//uniform
layout (set = 1, binding = 0) uniform sampler2D u_diffuse_texture;
layout (set = 1, binding = 1) uniform sampler2D u_sdf_texture;


layout (set = 1, binding = 2) uniform Block
{
    vec4 u_palette_info;    // x, y : per pixel uv of an atlas( pallete ) texture
} block;
                                //    z : 0 : stroke, 1: line body

//varying
layout (location = 5) in vec2 v_sdf_uv;
layout (location = 9) in vec2 v_dash_uv;
layout (location = 7) in vec4 v_diffuse;

layout (location = 4) in vec4 v_line_info;   // x : boundary distance field value
                            // y : pixel distance
                            // z : original period
                            // w : period with width weight

//out fragColor
layout (location = 0) out vec4 outFragColor;

const float end_edge = 0.98431752549;

void main()
{
    float pattern_length = abs( ( mod( v_line_info.y, v_line_info.w ) / v_line_info.w ) * v_line_info.z );
    vec2 pattern_uv = vec2( v_dash_uv.x + pattern_length * block.u_palette_info.x, v_dash_uv.y );
    float pattern_weight = texture( u_diffuse_texture, pattern_uv ).a;

    float sdf_value = texture( u_sdf_texture, v_sdf_uv.xy ).x;
    float alpha = 1.0 - smoothstep( v_line_info.x, end_edge, sdf_value );
    alpha *= min( 1.0, pattern_weight + ( 1.0 - block.u_palette_info.z ) );
    outFragColor = vec4( v_diffuse.rgb, alpha * v_diffuse.a );
//    gl_FragColor = vec4( 1.0, 0.0, 0.0, 1.0 );
}
